@@ -1,6 +1,6 @@
 // ------------------------------------------------------------------------
 // eca-chain.cpp: Class representing an abstract audio signal chain.
-// Copyright (C) 1999-2009 Kai Vehmanen
+// Copyright (C) 1999-2009,2012 Kai Vehmanen
 // Copyright (C) 2005 Stuart Allie
 //
 // Attributes:
@@ -62,13 +62,19 @@ class CHAIN : public ECA_AUDIO_POSITION {
   bool is_muted(void) const { return muted_rep; }
 
   /**
-   * Is processing enabled? If disabled, all chain operators
-   * will be skipped during processing. 
-   */
-  bool is_processing(void) const { return sfx_rep; }
+   * Is chain muted? If muted, audio buffers are zeroed during
+   * processing.
+   */ 
+  bool is_operator_bypassed(int op_index) const;
 
-  void toggle_muting(bool v) { muted_rep = v; }
-  void toggle_processing(bool v) { sfx_rep = v; }
+  /**
+   * Is processing enabled and bypassed? If bypassed, all chain
+   * operators will be skipped during processing. 
+   */
+  bool is_bypassed(void) const { return bypass_rep; }
+
+  void set_mute(int muted);
+  void set_bypass(int state);
 
   std::string name(void) const { return chainname_rep; }
   void name(const std::string& c) { chainname_rep = c; }
@@ -121,12 +127,14 @@ class CHAIN : public ECA_AUDIO_POSITION {
   void add_controller(GENERIC_CONTROLLER* gcontroller);
   void remove_chain_operator(int op_index);
 
+  void bypass_operator(int op_index, int bypassed);
+
   void set_parameter(int op_index, int param_index, CHAIN_OPERATOR::parameter_t value);
 
   int number_of_chain_operators(void) const { return chainops_rep.size(); }
   int number_of_chain_operator_parameters(int index) const;
 
-  const CHAIN_OPERATOR* get_chain_operator(int index) const { return chainops_rep[index]; }
+  const CHAIN_OPERATOR* get_chain_operator(int index) const { return chainops_rep[index].cop; }
   const GENERIC_CONTROLLER* get_controller(int index) const { return gcontrollers_rep[index]; }
 
   int number_of_controllers(void) const { return gcontrollers_rep.size(); }
@@ -203,14 +211,22 @@ class CHAIN : public ECA_AUDIO_POSITION {
 
  private:
 
+  bool is_valid_op_index(int op_index) const;
+
+  class COP_CONTAINER {
+  public:
+    CHAIN_OPERATOR* cop;
+    bool bypassed;
+  };
+
   bool initialized_rep;
   std::string chainname_rep;
   bool muted_rep;
-  bool sfx_rep;
+  bool bypass_rep;
   int in_channels_rep;
   int out_channels_rep;
 
-  std::vector<CHAIN_OPERATOR*> chainops_rep;
+  std::vector<struct COP_CONTAINER> chainops_rep;
   std::vector<GENERIC_CONTROLLER*> gcontrollers_rep;
 
   GENERIC_CONTROLLER* selected_controller_repp;
