@@ -1,6 +1,6 @@
 // ------------------------------------------------------------------------
 // eca-chainsetup.cpp: Class representing an ecasound chainsetup object.
-// Copyright (C) 1999-2006,2008,2009,2011 Kai Vehmanen
+// Copyright (C) 1999-2006,2008,2009,2011,2012 Kai Vehmanen
 // Copyright (C) 2005 Stuart Allie
 //
 // Attributes:
@@ -873,10 +873,7 @@ void ECA_CHAINSETUP::toggle_chain_muting(void)
   for(vector<string>::const_iterator a = selected_chainids.begin(); a != selected_chainids.end(); a++) {
     for(vector<CHAIN*>::iterator q = chains.begin(); q != chains.end(); q++) {
       if (*a == (*q)->name()) {
-	if ((*q)->is_muted()) 
-	  (*q)->toggle_muting(false);
-	else 
-	  (*q)->toggle_muting(true);
+	(*q)->set_mute(-1);
       }
     }
   }
@@ -896,10 +893,7 @@ void ECA_CHAINSETUP::toggle_chain_bypass(void)
   for(vector<string>::const_iterator a = selected_chainids.begin(); a != selected_chainids.end(); a++) {
     for(vector<CHAIN*>::iterator q = chains.begin(); q != chains.end(); q++) {
       if (*a == (*q)->name()) {
-	if ((*q)->is_processing()) 
-	  (*q)->toggle_processing(false);
-	else 
-	  (*q)->toggle_processing(true);
+	(*q)->set_bypass(-1);
       }
     }
   }
@@ -2001,6 +1995,61 @@ bool ECA_CHAINSETUP::execute_edit(const chainsetup_edit_t& edit)
 
   switch(edit.type)
     {
+    case edit_c_bypass:
+      {
+	if (edit.m.c_bypass.chain < 1 ||
+	    edit.m.c_bypass.chain > static_cast<int>(chains.size())) {
+	  retval = false;
+	  break;
+	}
+	CHAIN *ch = chains[edit.m.c_bypass.chain - 1];
+	ch->set_bypass(edit.m.c_bypass.val);
+	break;
+      }
+
+    case edit_c_muting:
+      {
+	if (edit.m.c_muting.chain < 1 ||
+	    edit.m.c_muting.chain > static_cast<int>(chains.size())) {
+	  retval = false;
+	  break;
+	}
+	CHAIN *ch = chains[edit.m.c_muting.chain - 1];
+	ch->set_mute(edit.m.c_muting.val);
+	break;
+      }
+
+#if NOT_YET_IMPLEMENTED
+    case edit_cop_remove:
+    case edit_ctrl_remove:
+      // pass through on purpose
+#endif
+
+    case edit_cop_add:
+    case edit_ctrl_add:
+      {
+        if ((edit.m.c_generic_param.chain < 1) ||
+             (edit.m.c_generic_param.chain > static_cast<int>(chains.size()))) {
+          retval = false;
+          break;
+        }
+        bool locked = is_locked_rep;
+        is_locked_rep = false;
+        const string& params = edit.param;
+        if (params.size() > 0 && params[0] == '-')
+          cparser_rep.interpret_object_option(params);
+        else
+          cparser_rep.interpret_object_option(string("-") + edit.param);
+        if (interpret_result() != true) {
+          ECA_LOG_MSG(ECA_LOGGER::errors,
+                      "cop-add error " + 
+                      interpret_result_verbose());
+          retval = false;
+        }
+        is_locked_rep = locked;
+        break;
+      }
+
     case edit_cop_set_param:
       {
 	if (edit.m.cop_set_param.chain < 1 ||
@@ -2014,6 +2063,20 @@ bool ECA_CHAINSETUP::execute_edit(const chainsetup_edit_t& edit)
 			  edit.m.cop_set_param.value);
 	break;
       }
+
+    case edit_cop_bypass:
+      {
+	if (edit.m.cop_bypass.chain < 1 ||
+	    edit.m.cop_bypass.chain > static_cast<int>(chains.size())) {
+	  retval = false;
+	  break;
+	}
+	CHAIN *ch = chains[edit.m.cop_bypass.chain - 1];
+	ch->bypass_operator(edit.m.cop_bypass.op, 
+			    edit.m.cop_bypass.bypass);
+	break;
+      }
+
     case edit_ctrl_set_param:
       {
 	if (edit.m.ctrl_set_param.chain < 1 ||
@@ -2177,7 +2240,7 @@ SAMPLE_SPECS::sample_pos_t ECA_CHAINSETUP::seek_position(SAMPLE_SPECS::sample_po
   ECA_LOG_MSG(ECA_LOGGER::user_objects,
 	      "seek position, chainsetup \"" +
 	      name() +
-	      "\" to pos in sec " + 
+	      "\" to pos in samples " + 
 	      kvu_numtostr(pos) + ".");
 
   if (is_enabled() == true) {
