@@ -1,6 +1,6 @@
 // ------------------------------------------------------------------------
 // eca-object-factory_test.h: Unit test for ECA_OBJECT_FACTORY
-// Copyright (C) 2002,2008,2009 Kai Vehmanen
+// Copyright (C) 2002,2008,2009,2012 Kai Vehmanen
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #include "audioio-device.h"
 #include "eca-chainop.h"
 #include "audiofx_ladspa.h"
+#include "audiofx_lv2.h"
 #include "preset.h"
 #include "generic-controller.h"
 #include "midiio.h"
@@ -92,8 +93,7 @@ void ECA_OBJECT_FACTORY_TEST::test_map(const ECA_OBJECT_MAP& objmap)
   while(p != regobjs.end()) {
     const T* obj = dynamic_cast<const T*>(objmap.object(*p));
     if (obj == 0) {
-      ECA_TEST_FAILURE("Unable to create object of type \"" + obj->name()
-		       + "\" from keyword \"" + *p + "\"");
+      ECA_TEST_FAILURE("Unable to create object from keyword \"" + *p + "\"");
     }
     else {
       ECA_LOG_MSG(ECA_LOGGER::user_objects, 
@@ -203,6 +203,11 @@ void ECA_OBJECT_FACTORY_TEST::do_run(void)
       ladspa_id_map.registered_objects().size()) {
       ECA_TEST_FAILURE("LADSPA plugin name and id maps have different number of plugins!");
   }
+  
+  ECA_LOG_MSG(ECA_LOGGER::info, "libecasound_tester: object factories - LV2-map");
+
+  ECA_OBJECT_MAP& lv2_map = ECA_OBJECT_FACTORY::lv2_plugin_map();
+  test_map<EFFECT_LV2>(lv2_map);
 
   ECA_LOG_MSG(ECA_LOGGER::info, "libecasound_tester: object factories - preset-map");
 
